package com.steerinfo.dil.controller;

import com.alibaba.fastjson.JSON;
import com.steerinfo.dil.feign.ESFeign;
import com.steerinfo.dil.mapper.RmsOilTypeMapper;
import com.steerinfo.dil.model.RmsCarDriver;
import com.steerinfo.dil.service.IRmsOilPriceService;
import com.steerinfo.dil.service.impl.RmsCarDriverServiceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.dil.model.RmsOilPrice;
import com.steerinfo.dil.service.IRmsOilPriceService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.*;
import java.math.BigDecimal;

/**
 * RmsOilPricea RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-11-23 05:01
 * 类描述
 * 修订历史：
 * 日期：2021-11-23
 * 作者：generator
 * 参考：
 * 描述：RmsOilPricea RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
/*@RestController
@RequestMapping("/${api.version}/rmsoilprices")
public class RmsOilPriceController extends BaseRESTfulController {

    @Autowired
    IRmsOilPriceService rmsOilPriceService;


}*/




@RestController
@RequestMapping("/${api.version}/rmsoilprice")
public class RmsOilPriceController extends BaseRESTfulController {
    @Autowired
    IRmsOilPriceService rmsOilPriceService;
    @Autowired
    ColumnDataUtil columnDataUtil;
    @Autowired
    ESFeign esFeign;
    @Autowired
    RmsOilTypeMapper rmsOilTypeMapper;

    /**
     *增加新油价
     * @param
     * @return
     */
    @ApiOperation(value="创建", notes="根据RmsOilPrice对象创建")
    @ApiImplicitParam(name = "rmsOilPrice", value = "详细实体rmsOilPrice", required = true, dataType = "RmsOilPrice")
    @PostMapping(value = "/insertOilPrice")
    public RESTfulResult insertOilPrice(@RequestBody(required = false) Map<String, Object> map){
        int result = rmsOilPriceService.insertOilPrice(map);
        return success(result);
    }

    @ApiOperation(value="创建", notes="添加油品名称")
    @ApiImplicitParam(name = "oilTypeName", value = "油品名称", required = true, dataType = "String")
    @PostMapping(value = "/addOilType")
    public RESTfulResult addOilType(String oilTypeName){
        int result = rmsOilPriceService.addOilType(oilTypeName);
        return success(result);
    }

    @ApiOperation(value="创建", notes="查询油品名称")
    @PostMapping(value = "/oilNameSelect")
    public RESTfulResult oilNameSelect(){
        List<Map<String, Object>> maps = rmsOilTypeMapper.oilNameSelect();
        return success(maps);
    }
    /**
     *更新油价
     * @param
     * @return
     */


    @PostMapping(value = "/updateOilPrice")
    public RESTfulResult updateOilPrice(@RequestBody(required = false) Map<String, Object> mapValue){
        int result = rmsOilPriceService.updateOilPrice(mapValue);
        return success(result);
    }

    //淘汰版本仅供参考
/*    @PostMapping(value = "/updateOilPriceById/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult updateOilPriceById(@RequestBody RmsOilPrice rmsOilPrice){
        int result = rmsOilPriceService.updateOilPrice(rmsOilPrice);
        return success(result);
    }*/


    /**
     *删除油价
     * @param id
     * @return
     */
    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal")
    @PostMapping(value = "/deleteOilPrice/{id}")
    public RESTfulResult deleteOilPrice(@PathVariable("id") BigDecimal id){
        int i = rmsOilPriceService. deleteOilPrice(id);
        if(i <= 0){
            return failed();
        }
        return success();
    }

    /**
     * 根据id获取油价
     * @param id
     * @return
     */
    //测试成功

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal")
    @PostMapping(value = "/getOilPriceById/{id}")
    public RESTfulResult getOilPriceById(@PathVariable("id") BigDecimal id){
        Map<String,Object> map= rmsOilPriceService.getOilPriceById(id);
        return success(map);
    }


    /**
     * 油价类型下拉框
     * */
/*//未实现，保留
    @GetMapping("/getOilPriceTypeId")
    public RESTfulResult getOilPriceTypeId(){
        return success(rmsOilPriceService.getOilPriceTypeId());
    }
*/


    @ApiOperation(value = "油价模糊查询", notes = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "", required = false, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/getOilPriceResultList")
    public RESTfulResult getOilPriceResultList(@RequestBody(required = false) Map<String, Object> mapValue,
                                                Integer pageNum,
                                                Integer pageSize,
                                                Integer apiId,
                                                String con) throws ParseException {
        if (mapValue==null){
            mapValue=new HashMap<>();
        }
        if(con != null && !"".equals(con) && !"null".equals(con)){
            //对包含#以及%的数据进行解析操作
            if(con.contains("%")){
                if (con.contains("%23")){
                    String replace = con.replace("%23", "#");
                    mapValue.put("index", "%"+replace+"%");
                }else {
                    NumberFormat numberFormat = NumberFormat.getPercentInstance();
                    Number parse = numberFormat.parse(con);
                    mapValue.put("index", "%"+parse+"%");
                }
            }else {
                if (con.contains("启用")||con.contains("停用")){
                    mapValue.put("index1", "%"+con+"%");
                }else {
                    mapValue.put("index", "%"+con+"%");
                }
            }
        }
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = rmsOilPriceService.getOilPriceList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, null, columnList);
        return success(data);
    }

}